﻿using System;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region HTTP 클라이언트 구하기 - GetHTTPClient(baseAddress)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="baseAddress">기준 주소</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(string baseAddress)
        {
            HttpClient client = new HttpClient();

            client.BaseAddress = new Uri(baseAddress);

            client.DefaultRequestHeaders.Add("APIKey", "A7A54AD3-91E1-4C3A-98C6-A17E39C9EA3D");

            client.DefaultRequestHeaders.Accept.Clear();

            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }

        #endregion
        #region 파일 업로드하기 (비동기) - UploadFileAsync(client, filePath)

        /// <summary>
        /// 파일 업로드하기 (비동기)
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="filePath">파일 경로</param>
        /// <returns>HTTP 응답 메시지 태스크</returns>
        private static async Task<HttpResponseMessage> UploadFileAsync(HttpClient client, string filePath)
        {
            string url = string.Format("/api/File/Upload");

            client.DefaultRequestHeaders.Add("FileName", Path.GetFileName(filePath));

            using(FileStream sourceStream = new FileStream(filePath, FileMode.Open))
            {
                StreamContent content = new StreamContent(sourceStream);

                HttpResponseMessage httpResponseMessage = await client.PostAsync(url, content);

                return httpResponseMessage;
            }
        }

        #endregion
        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        static void Main()
        {
            Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");
            Console.WriteLine("--------------------------------------------------");
            Console.ReadKey(true);

          //using(HttpClient client = GetHTTPClient("https://localhost:44362")) // IIE Express 사용시
            using(HttpClient client = GetHTTPClient("https://localhost:5001"))
            {
                // 파일 크기는 2GB까지만 가능하다.
                var response = UploadFileAsync(client, @"d:\sample.mp4").GetAwaiter().GetResult();

                Console.WriteLine(response.StatusCode);
                Console.WriteLine(response.ReasonPhrase);
            }

            Console.WriteLine("--------------------------------------------------");
            Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");
            Console.ReadKey(true);
        }

        #endregion
    }
}