﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Threading.Tasks;

namespace TestServer
{
    /// <summary>
    /// API 키 어트리뷰트
    /// </summary>
    [AttributeUsage(validOn: AttributeTargets.Class)]
    public class APIKeyAttribute : Attribute, IAsyncActionFilter
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// API 키명
        /// </summary>
        public static readonly string API_KEY_NAME = "APIKey";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 액션 실행시 처리하기 (비동기) - OnActionExecutionAsync(context, nextAction)

        /// <summary>
        /// 액션 실행시 처리하기 (비동기)
        /// </summary>
        /// <param name="context">액션 실행 컨텍스트</param>
        /// <param name="nextAction">다음 액션 실행 대리자</param>
        /// <returns>태스크</returns>
        public async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate nextAction)
        {
            if(!context.HttpContext.Request.Headers.TryGetValue(API_KEY_NAME, out var sourceAPIKey))
            {
                context.Result = new ContentResult()
                {
                    StatusCode = 401,
                    Content    = "API Key was not provided."
                };

                return;
            }

            IConfiguration configuration = context.HttpContext.RequestServices.GetRequiredService<IConfiguration>();

            string targetAPIKey = configuration.GetValue<string>(API_KEY_NAME);

            if(!targetAPIKey.Equals(sourceAPIKey))
            {
                context.Result = new ContentResult()
                {
                    StatusCode = 401,
                    Content    = "API Key is not valid."
                };

                return;
            }

            await nextAction();
        }

        #endregion
    }
}