﻿using Microsoft.AspNetCore.Mvc;
using System.IO;
using System.Threading.Tasks;

namespace TestServer.Controllers
{
    /// <summary>
    /// 파일 컨트롤러
    /// </summary>
    [Route("api/[controller]/[action]")]
    [ApiController]
    [APIKey]
    public class FileController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 최대 파일 크기
        /// </summary>
        private const long MAXIMUM_FILE_SIZE = 10737418240L; // 10GB

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 업로드하기 - Upload()

        /// <summary>
        /// 업로드하기
        /// </summary>
        /// <returns>액션 결과 태스크</returns>
        [HttpPost]
        [RequestSizeLimit(MAXIMUM_FILE_SIZE)]
        [RequestFormLimits(MultipartBodyLengthLimit = MAXIMUM_FILE_SIZE)]
        public async Task<IActionResult> Upload()
        {
            using(Stream sourceStream = Request.Body)
            {
                string fileName = Request.Headers["FileName"];

                string targetFilePath = Path.Combine("d:\\Download", fileName);

                if(!System.IO.File.Exists(targetFilePath))
                {
                    using(FileStream targetStream = new FileStream(targetFilePath, FileMode.Create))
                    {
                        await sourceStream.CopyToAsync(targetStream);
                    }

                    return new JsonResult(new { FilePath = targetFilePath });
                }

                // 파일이 존재하는 경우 잘못된 요청을 반환한다.
                return BadRequest();
            }
        }

        #endregion
    }
}